<?php

/**
 * FiloBlu Controller for Catalor Search
 *
 * @category    FiloBlu
 * @package     FiloBlu_AjaxLayeredNavigation
 * @author      Vincenzo Assenza
 *
 */

namespace FiloBlu\AjaxLayeredNavigation\Controller\CatalogSearch\Result;

use Magento\Catalog\Model\Layer\Resolver;
use Magento\Catalog\Model\Session;
use Magento\CatalogSearch\Helper\Data;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\View\Result\PageFactory;
use Magento\Search\Model\QueryFactory;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 *
 */
class Index extends \Magento\CatalogSearch\Controller\Result\Index
{

    protected $resultJsonFactory;
    protected $resultPageFactory;
    protected $_catalogLayer;

    /**
     * Catalog session
     *
     * @var Session
     */
    protected $_catalogSession;

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var QueryFactory
     */
    private $_queryFactory;

    /**
     * Catalog Layer Resolver
     *
     * @var Resolver
     */
    private $layerResolver;

    /**
     * @param Context $context
     * @param Session $catalogSession
     * @param StoreManagerInterface $storeManager
     * @param QueryFactory $queryFactory
     * @param Resolver $layerResolver
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     */
    public function __construct(
        Context $context,
        Session $catalogSession,
        StoreManagerInterface $storeManager,
        QueryFactory $queryFactory,
        Resolver $layerResolver,
        PageFactory $resultPageFactory,
        JsonFactory $resultJsonFactory
    ) {
        $this->_storeManager = $storeManager;
        $this->_catalogSession = $catalogSession;
        $this->_queryFactory = $queryFactory;
        $this->layerResolver = $layerResolver;
        parent::__construct($context, $catalogSession, $storeManager, $queryFactory, $layerResolver);
        $this->resultPageFactory = $resultPageFactory;
        $this->resultJsonFactory = $resultJsonFactory;
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json|void|null
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function execute()
    {
        if ($this->_request->getParam('is_ajax') &&
            $this->getRequest()->isXmlHttpRequest() &&
            $this->_objectManager->get(ScopeConfigInterface::class)->getValue(
                'filoblu_ajaxlayerednavigation/general/enabled',
                ScopeInterface::SCOPE_STORE
            ) == 1) {
            return $this->_viewAjax();
        }

        return parent::execute();
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json|void
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    private function _viewAjax()
    {
        $this->layerResolver->create(Resolver::CATALOG_LAYER_SEARCH);
        $query = $this->_queryFactory->get();

        $query->setStoreId($this->_storeManager->getStore()->getId());

        if ($query->getQueryText() != '') {
            if ($this->_objectManager->get(Data::class)->isMinQueryLength()) {
                $query->setId(0)->setIsActive(1)->setIsProcessed(1);
            } else {
                $query->saveIncrementalPopularity();

                if ($query->getRedirect()) {
                    $this->getResponse()->setRedirect($query->getRedirect());
                    return;
                }
            }

            $this->_objectManager->get(Data::class)->checkNotes();

            //ajax response
            return $this->_jsonResponse();
        }

        $this->_redirectReferer();
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    private function _jsonResponse()
    {
        $response = [];

        $blockList = unserialize(
            $this->_objectManager->get(ScopeConfigInterface::class)->getValue(
                'filoblu_ajaxlayerednavigation/general/block_list',
                ScopeInterface::SCOPE_STORE
            )
        );

        $page = $this->resultPageFactory->create();

        foreach ($blockList as $singleBlock) {
            $singleXmlName = trim($singleBlock['xml_name']);
            $singleDomName = $singleBlock['dom_name'];
            $block = $page->getLayout()->getBlock($singleXmlName);
            if ($block) {
                $response[$singleDomName] = $block->toHtml();
            }
        }

        return $this->resultJsonFactory->create()->setData($response);
    }
}
